#!/usr/bin/python -u
#
# simple wrapper for postgres insert statement.
#

"""insert new record into ETS. For use in ADIT-NMR"""

import os
import sys
import datetime
import psycopg2
import psycopg2.extensions
import subprocess
import pprint
import ConfigParser
from optparse import OptionParser

PSQL = "/usr/bin/psql"

CONFFILE = os.path.join( os.path.split( __file__ )[0], "insert_nmrif_to_ets.conf" )

# production
#
#URL = {
#    "database" : "ETS",
#    "user"     : "ets",
#    "host"     : "ets.bmrb.wisc.edu"
#}

# test
#
#URL = {
#    "psql"     : "/usr/bin/psql",
#    "database" : "ETS_TEST",
#    "user"     : "etstest",
#    "host"     : "pike.bmrb.wisc.edu"
#}

# disabled
#
URL = {}

if __name__ == "__main__" :

    usage = "usage: %prog [options]"
    op = OptionParser( usage = usage )
    op.add_option( "-v", "--verbose", action = "store_true", dest = "verbose",
        default = False, help = "print lots of messages to stdout" )
    op.add_option( "-n", "--dry-run", action = "store_false", dest = "run",
        default = True, help = "don't run the query" )

    op.add_option( "-a", "--dep_id", action = "store", type = "string",
        dest = "aditid", help = "deposition ID" )
    op.add_option( "-b", "--bmrb_id", action = "store", type = "string",
        dest = "bmrbid", help = "BMRB ID" )
    op.add_option( "-r", "--restart_id", action = "store", type = "string",
        dest = "sessid", help = "ADIT-NMR restart ID" )

    op.add_option( "-c", "--contact_one", action = "store", type = "string",
        dest = "cp1", help = "contact person 1 name" )
    op.add_option( "-d", "--contact_two", action = "store", type = "string",
        dest = "cp2", help = "contact person 2 name" )
    op.add_option( "-e", "--email", action = "store", type = "string",
        dest = "email", help = "contact e-mail address(es)" )

    op.add_option( "-t", "--title", action = "store", type = "string",
        dest = "title", help = "molecule name (title)" )

    op.add_option( "-s", "--hold_status", action = "store", type = "string",
        dest = "hold", help = "hold status" )

    (options, args) = op.parse_args()

# defaults
#
    today = datetime.date.today().isoformat()
    params = {  "source" : "Author",
                "submit_type" : "Dep",
                "status" : "nd",
                "lit_search_required" : "N",
                "submission_date" : today,
                "accession_date" : today,
                "last_updated" : today
    }

    if options.hold is not None : 
        params["onhold_status"] = options.hold
    else : 
        params["onhold_status"] = "Pub"

    if options.aditid is not None :
        params["nmr_dep_code"] = options.aditid

    if options.bmrbid is not None :
        params["bmrbnum"] = options.bmrbid

    if options.sessid is not None :
        params["restart_id"] = options.sessid

    if options.cp1 is not None :
        params["contact_person1"] = options.cp1

    if options.cp2 is not None :
        params["contact_person2"] = options.cp2

    if options.email is not None :
        params["author_email"] = options.email

    if options.title is not None :
        params["molecular_system"] = options.title

# config
#
    cp = ConfigParser.SafeConfigParser( defaults = URL )
    confile = None
    if len( args ) > 0 : conffile = args[0]
    else : conffile = CONFFILE
    if os.path.exists( conffile ) :
        cp.read( conffile )

#    colstr = ",".join( i for i in params.keys() )
#    valstr = ",%s" * len( cols )
#    sql = "insert into entrylog (depnum,%s) values (nextval('depnum_seq')%s)" % (colstr, valstr)

# we're having problems with psycopg2 failing to connect for no apparent reason
#
    colstr = ""
    valstr = ""
    have_id = False
    for (key, val) in params.items() :
        colstr += key + ","
        if key == "bmrbnum" :
            valstr += val + ","
            have_id = True
        else : valstr += psycopg2.extensions.adapt( val ).getquoted() + ","
    colstr = colstr[:-1]
    valstr = valstr[:-1]
    sql = "insert into entrylog (depnum,%s) values (nextval('depnum_seq'),%s)" % (colstr, valstr)

# ETS doesn't have a trigger, this needs to be done manually
#
    log = "insert into logtable (logid,depnum,actdesc,newstatus,statuslevel,logdate,login) " \
        + "values (nextval('logid_seq'),currval('depnum_seq'),'NEW DEPOSITION','nd',1,now(),'')"

# psql can run 2 commands in single call/transaction
#
    sql = sql + " ; " + log

# log SQL always
#
    sys.stdout.write( sql )
    sys.stdout.write( "\n" )

    psql = None
    database = None
    user = None
    host = None
    if cp.has_section( "ets" ) :
        if cp.has_option( "ets", "psql" ) :
            psql = cp.get( "ets", "psql" )
        if cp.has_option( "ets", "database" ) :
            database = cp.get( "ets", "database" )
        if cp.has_option( "ets", "host" ) :
            host = cp.get( "ets", "host" )
        if cp.has_option( "ets", "user" ) :
            user = cp.get( "ets", "user" )

#    print psql,host,database,user

    if (psql is None) or (database is None) or (host is None) or (user is None) :
        sys.exit( 0 )

    if not os.path.exists( psql ) :
        sys.stderr.write( "Postgresql is not installed!" )
        sys.exit( 1 )

    cmd = [psql, "-h", host, "-d", database, "-U", user, "-c", sql]

    if options.verbose : pprint.pprint( cmd )

    if options.run :
        if not have_id :
            sys.stderr.write( "No BMRB ID!" )
            sys.exit( 1 )

        p = subprocess.Popen( cmd )
        p.wait()
        if p.returncode != 0 :
            sys.stderr.write( "psql returned %d\n" % (p.returncode,) )

    sys.exit( 0 )
#
#
#
